/*
 * Programming in Java, Jacobs University, 2015
 *
 * This file demonstrates how an iterator and an iterable can be be
 * implemented. The class Range represents an iterable range of
 * numbers that can be iterated over in a for loop. The Range class
 * has a nested RangeIterator class implementing the corresponding
 * iterator.
 *
 * Simplified definition of the interfaces:
 *
 * public interface Iterable<T>
 * {
 *     Iterator<T> iterator();
 * }
 *
 * public interface Iterator<E>
 * {
 *     boolean hasNext();
 *     E next();
 *     void remove();
 * }
 *
 * A class implementing the Iterable interface can be used in the
 * compact for-loop notation.
 */

import java.util.NoSuchElementException;
import java.util.Iterator;

public class Range implements Iterable<Integer>
{
    private int start, end;

    public Range(int start, int end)
    {
	this.start = start;
	this.end = end;
    }

    public Iterator<Integer> iterator()
    {
	 return new RangeIterator();
    }

    private class RangeIterator implements Iterator<Integer>
    {
	private int cursor;
	
	public RangeIterator()
	{
	    this.cursor = Range.this.start;
	}
	
	public boolean hasNext()
	{
	    return this.cursor < Range.this.end;
	}
	
	public Integer next()
	{
	    if (! this.hasNext()) {
		throw new NoSuchElementException();
	    }
	    return cursor++;
	}
	
	public void remove()
	{
	    throw new UnsupportedOperationException();
	}
    }

    public static void main(String[] args)
    {
	for (Integer i: new Range(1, 10)) {
	    System.out.println(i);
	}
    }
}
